#!/bin/bash
#
# Copyright (C) 2021 Actiphy, Inc.
#   All rights reserved.
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Actiphy, Inc.
# The copyright notice above does not evidence any actual or intended
# publication of such source code.
#
# File:  ptov_pre.sh
# Ver:	1.0.12
# Date:  10/21/21
#
# Created by Yoshinobu Ito Actiphy,INC.
#
# Qualified Platforms:
#	Redhat 5.x or later
#	CentOS 5.x or later
#	Oracle Linux 5.x or later
#	Open Suse 12.1 or later
#	Suse Enterprise Linux 11SP3 or later
#	Ubuntu 12.04 or later
#	Debian10.x
#
# Requirements:
#	AIP 2018 Update5 HyperAgent or later
#

export LANG=C
script_ver=1.0.12

if [ ! -e /opt/Actiphy/logs ]; then
	mkdir -p /tmp/PtoV
	LOG=/tmp/PtoV/ptov_`date +%Y%m%d%H%M%S`.log
else
	mkdir -p /opt/Actiphy/logs/PtoV
	LOG=/opt/Actiphy/logs/PtoV/ptov_`date +%Y%m%d%H%M%S`.log
fi

#FIFO=/tmp/$$.fifo
#mkfifo ${FIFO}
#trap '/bin/rm -f ${FIFO}' exit
#tee ${LOG} < ${FIFO} &
#exec > ${FIFO} 2>&1

echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Script version "$script_ver >> $LOG

curr_dir=$(dirname $(readlink -f $0))

kernel_ver=`uname -r | sed -e "s/^\([0-9]\).*$/\1/"`		
echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** kernel version "$kernel_ver >> $LOG

kernel_patch=`uname -r | sed -e "s/^[0-9]*.[0-9]*.[0-9]*-\([0-9]*\).*$/\1/"`
echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** kernel patch level "$kernel_patch >> $LOG

Sync_cmd (){
sleep 3
sync
sync
sync
sleep 3
}

Check_Dist(){

if [ -f /etc/os-release ]; then
	DIST=`cat /etc/os-release | grep -e "^NAME" | sed -e 's/^NAME=//' -e 's/^\"//' -e 's/$*"//' | awk -F " " '{print $1}'`
	OSVER=`cat /etc/os-release | grep -e "^VERSION_ID" | sed -e 's/^VERSION_ID="//' -e 's/^VERSION_ID=//' -e 's/$*"//' -e s'/\.[0-9]//'`
else
	if [ ! -e /etc/system-release ]; then #SLES11.x 
		str=`find /etc -name *-release | grep SuSE | wc -l`
		if [ $str -eq 1 ]; then
			OSVER=`cat /etc/SuSE-release | grep VERSION | awk '{print $3}'`
			DIST=`cat /etc/SuSE-brand | head -n 1`
		fi
	else
		DIST=`cat /etc/system-release | awk -F " " '{print $1}'`
		OSVER=`cat /etc/system-release | grep -oE "([0-9]\.[0-9]*)" | sed -e s'/\.[0-9]*//'`	
	fi
fi

# Debian10.x, ubuntu12.x or later does not need to use ptov script.
case "$DIST" in
	"Red") OSDIST="RHEL" ;;
	"CentOS") OSDIST="CentOS" ;;
	"Asianux") OSDIST="Asianux" ;;
	"Oracle") OSDIST="Oracle" ;;
	"AlmaLinux") OSDIST="AlmaLinux" ;;
	"Rocky") OSDIST="Rocky" ;;
	"openSUSE") OSDIST="openSUSE" ;;
	"SLES") OSDIST="SLES" ;;
	"SLED") OSDIST="SLED" ;;
#	"Debian") OSDIST="Debian" ;;
#	"Ubuntu") OSDIST="Ubuntu" ;;
	"Amazon") OSDIST="Amazon" ;;
	*) echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** [$DIST] does not need to script." >> $LOG 
	   exit 0
esac

}
Check_Dist

Check_initrd(){

if [ "$OSDIST" = "openSUSE" -o "$OSDIST" = "SLES" -o  "$OSDIST" = "SLED" ]; then
	initrd_file=`ls /boot | grep -v kdump | grep -E initrd-$(uname -r)$`
else 
	initrd_file=`ls /boot | grep -v kdump | grep -E $(uname -r).img$`
fi
echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** initrd file ["$initrd_file"]" >> $LOG 
try_cmd=`lsinitrd -h 2>&1 | grep "Usage" | wc -l`
if [ $try_cmd -ge 1 ]; then
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Exist lsinitrd command." >> $LOG
	lsinit_cmd=lsinitrd
else
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Not exist lsinitrd command." >> $LOG
	lsinit_cmd=lsinitramfs
fi

}

Check_dracut(){

cmd=`dracut -h 2>&1 | grep "Usage" | wc -l`
if [ $cmd -ge 1 ]; then
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Exist dracut command." >> $LOG  
	dracut_cmd=1	#exist dracut
else
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Not exist dracut command." >> $LOG
	dracut_cmd=0	#not exist dracut
fi
}

Check_initramfstool(){

tools=/etc/initramfs-tools/modules
if [ -f $tools ]; then
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Exist initramfs-tools." >> $LOG  
	initramfs_tools=1	#exist initramfs-tools
else
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Not exist initramfs-tools." >> $LOG  
	initramfs_tools=0	#not exist initramfs-tools
fi

}

mkinitrd_cmd (){
INITRD_conf=/etc/sysconfig/kernel
INITRD_item=`cat $INITRD_conf | grep -E "^INITRD" | wc -l`
num_lines=`cat $INITRD_conf | grep -nE "^INITRD" | sed -e "s/^\([0-9]*\).*$/\1/"`

exist_driver_suse=`$lsinit_cmd /boot/$initrd_file | grep -e hv_storvsc -e mptsas -e vmw_pvscsi -e virtio_scsi -e xen-netfront | wc -l`
if [ $exist_driver_suse -le 1 ]; then
	if [ -f $INITRD_conf -a $INITRD_item -eq 1 ]; then	#suse11
		module=`cat $INITRD_conf | grep -E ^INITRD | sed 's/^.*"\(.*\)".*$/\1/'`
		mod_array+=($module)
		mod_array=(${mod_array[@]} ${array_dracut[@]})
		aa=( $( IFS=$'\n' ; echo "${mod_array[*]}" | sort | uniq ; ) )
		list=`echo ${aa[*]}`
		\cp -f $INITRD_conf /tmp/
		if [ $? -eq 0 ]; then
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful backup "$INITRD_conf >> $LOG
		else
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed backup "$INITRD_conf >> $LOG
			exit 1
		fi
		sed -i ''$num_lines"d" $INITRD_conf
		IFS_bk=$IFS
		IFS=$'\n'
		sed -i ''$num_lines"i"' INITRD_MODULES="'$list'"' $INITRD_conf
		IFS=$IFS_bk
		\cp -f /boot/$initrd_file /tmp/
			if [ $? -eq 0 ]; then
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful backup ["$initrd_file"]." >> $LOG
				mkinitrd >> $LOG 2>&1
				if [ $? -eq 0 ]; then
					echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful rebuild initrd." >> $LOG
					exist_driver1=`$lsinit_cmd /boot/$initrd_file | grep -e hv_storvsc -e mptsas -e vmw_pvscsi -e virtio_scsi -e xen-netfront | wc -l`
					if [ $exist_driver1 -ge 2 ]; then
						Sync_cmd
						echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful added driver." >> $LOG
					else
						echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed added driver." >> $LOG
					fi
				else
					echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed rebuild initrd." >> $LOG
					\cp -f /tmp/$initrd_file /boot/
					if [ $? -eq 0 ]; then
						echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Recovery ["$initrd_file"]." >> $LOG
					fi
					exit 1
				fi
			else
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed backup ["$initrd_file"]." >> $LOG
				exit 1
			fi
	fi
else
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Already exist driver in ["$initrd_file"]." >> $LOG
fi
}

initramfs_tools () {
mod_file=/etc/initramfs-tools/modules
exist_driver=`$lsinit_cmd /boot/$initrd_file | grep -v Arguments | grep -e hv_storvsc -e mptsas -e vmw_pvscsi -e virtio_scsi -e xen-netfront | wc -l`
if [ $exist_driver -ne 5 ]; then
	if [ -f $mod_file ]; then
		mod_str=`cat /etc/initramfs-tools/modules | grep -v "#"`
		mod_list=($mod_str)
		for line in "${mod_list[@]}"; do
			echo $line
		done
		mod_list=(${mod_list[@]} ${array_dracut[@]})
		bb=( $( IFS=$'\n' ; echo "${mod_list[*]}" | sort | uniq -u ; ) )
		list1=`echo ${bb[*]}`
		sed -i '$a''#### Added by ActiveImage Protector ####' $mod_file
		for i in ${list1[@]}
		do
			sed -i '$a'$i'' $mod_file
		done
		sed -i '$a''##################' $mod_file
		\cp -f /boot/$initrd_file /tmp/
		if [ $? -eq 0 ]; then
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful backup ["$initrd_file"]." >> $LOG
			update-initramfs -u >> $LOG 2>&1
			if [ $? -eq 0 ]; then
				Sync_cmd
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful rebuild initramfs." >> $LOG
 				exist_driver1=`$lsinit_cmd /boot/$initrd_file | grep -v Arguments | grep -e hv_storvsc -e mptsas -e vmw_pvscsi -e virtio_scsi -e xen-netfront | wc -l`
				if [ $exist_driver1 -eq 3 ]; then
					echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful added driver." >> $LOG
				else
					echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed added driver." >> $LOG
				fi
			else
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed rebuild initramfs." >> $LOG
				\cp -f /tmp/$initrd_file /boot/
				if [ $? -eq 0 ]; then
					echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Recovery ["$initrd_file"]." >> $LOG
				fi
				exit 1
			fi
			update-grub2 >> $LOG 2>&1
			if [ $? -eq 0 ]; then
				Sync_cmd
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful update grub2." >> $LOG
			else
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed update grub2." >> $LOG
			fi
		else
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed backup ["$initrd_file"]." >> $LOG
			exit 1
		fi
	fi
else
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Already exist driver in ["$initrd_file"]." >> $LOG
fi
}

Check_driver_3() {
exist_driver=`$lsinit_cmd /boot/$initrd_file | grep -v Arguments | grep -e hv_storvsc -e mptsas -e vmw_pvscsi -e virtio_scsi -e xen-netfront | wc -l`
if [ $exist_driver -ne 5 ]; then
	\cp -f /boot/$initrd_file /tmp/
	if [ $? -eq 0 ]; then
		echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful backup ["$initrd_file"]." >> $LOG
		dracut -v --add-drivers "${array_dracut}" --force >> $LOG 2>&1
		if [ $? -eq 0 ]; then
			Sync_cmd
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful rebuild initramfs." >> $LOG
 			exist_driver1=`$lsinit_cmd /boot/$initrd_file | grep -v Arguments | grep -e hv_storvsc -e mptsas -e vmw_pvscsi | wc -l`
			if [ $exist_driver1 -eq 3 ]; then
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful added driver." >> $LOG
			else
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed added driver." >> $LOG
			fi
		else
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed rebuild initramfs." >> $LOG
			\cp -f /tmp/$initrd_file /boot/
			if [ $? -eq 0 ]; then
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Recovery ["$initrd_file"]." >> $LOG
			fi
			exit 1
		fi
	else
		echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed backup ["$initrd_file"]." >> $LOG
		exit 1
	fi
else
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Already exist driver in ["$initrd_file"]." >> $LOG
fi
}

exist_driver_2(){
mkdir /tmp/init
\cp -f /boot/$initrd_file /tmp/init
cd /tmp/init
zcat $initrd_file | cpio -idv
exist_driver2=`ls lib | grep -e ${array_el5[0]} -e ${array_el5[1]} -e ${array_el5[2]} -e ${array_el5[3]} | wc -l`
rm -rf /tmp/init
}


Check_driver_2() {
if [ ! -f /etc/modprobe.conf.bak ]; then
	\cp -f /etc/modprobe.conf /etc/modprobe.conf.bak
	if [ -f /etc/modprobe.conf.bak ]; then
		echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful backup /etc/modprobe.conf.bak." >> $LOG
	fi
fi

str=`cat /etc/modprobe.conf | grep -E 'scsi_hostadapter' | awk '{print $2}' | rev | cut -c 1 | rev | tail -n1`
check_conf=`cat /etc/modprobe.conf | grep -e mptbase -e  mptspi -e ata_piix -e mptsas | wc -l`
cnt=`expr $str + 1`
if [ $? -ne 0 ]; then
	cnt=1
fi
if [ $check_conf -ne 4 ]; then
	for i in {0..3}
	do
		conf=`cat /etc/modprobe.conf`
		val=`echo $conf | grep ${array_el5[$i]}| wc -l`
		if [ $val -eq 0 ]; then
			sed -i '$a alias scsi_hostadapter'$cnt' '${array_el5[$i]}'' /etc/modprobe.conf
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Set ${array_el5[$i]} to modprobe.conf." >> $LOG
			cnt=`expr $cnt + 1`
		else
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Already set ${array_el5[$i]} to modprobe.conf." >> $LOG
		fi
	done
	\cp -f /boot/$initrd_file /tmp/
	if [ $? -eq 0 ]; then
		echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful backup ["$initrd_file"]." >> $LOG
		if [ $kernel_patch -gt 308 ]; then
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Enabeld HV Integration Services" >> $LOG
			mkinitrd -v -f --with=hid-base-hv --with=hid-hyperv --with=hv_utils --with=hv_vmbus --with=hv_storvsc --with=hv_netvsc /boot/$initrd_file $(uname -r) >> $LOG 2>&1
		else
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Not enabeld HV Integration Services" >> $LOG
			mkinitrd -v -f /boot/$initrd_file $(uname -r) >> $LOG 2>&1
		fi
		if [ $? -eq 0 ]; then
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful rebuild initrd." >> $LOG
			exist_driver_2
			if [ $exist_driver2 -eq 4 ]; then
				Sync_cmd
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Successful added driver." >> $LOG
			else
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed added driver." >> $LOG
			fi
		else
			echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed rebuild initrd." >> $LOG
			\cp -f /tmp/$initrd_file /boot/
			if [ $? -eq 0 ]; then
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Recovery ["$initrd_file"]." >> $LOG
			fi
			\mv -f /etc/modprobe.conf.bak /etc/modprobe.conf
			if [ ! -f /etc/modprobe.conf.bak ]; then
				echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Recovery /etc/modprobe.conf." >> $LOG
			fi
			exit 1
		fi
	else
		echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed backup ["$initrd_file"]." >> $LOG
		exit 1
	fi
else
	exist_driver_2
	if [ $exist_driver2 -eq 4 ]; then
		echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Already exist driver in ["$initrd_file"]." >> $LOG
	fi
fi
}

xorg(){

hv_conf=$(cat << EOS >> /boot/AIP_xorgconf/hyperv_xorg.conf.el5
# Xorg configuration created by pyxf86config

Section "ServerLayout"
	Identifier     "Default Layout"
	Screen      0  "Screen0" 0 0
	InputDevice    "Keyboard0" "CoreKeyboard"
EndSection

Section "ServerFlags"
	Option	    "AllowMouseOpenFail" "yes"
EndSection

Section "InputDevice"
	Identifier  "Keyboard0"
	Driver      "kbd"
EndSection

Section "Device"
	Identifier  "Videocard0"
	Driver      "vesa"
EndSection

Section "Screen"
	Identifier "Screen0"
	Device     "Videocard0"
	DefaultDepth     24
	SubSection "Display"
		Viewport   0 0
		Depth     24
	EndSubSection
EndSection
EOS
)

hv_suse11_conf=$(cat << EOS >> /boot/AIP_xorgconf/hyperv_xorg.conf.suse11
# Xorg configuration created by pyxf86config

Section "ServerLayout"
	Identifier     "Default Layout"
	Screen      0  "Screen0" 0 0
	InputDevice    "Keyboard0" "CoreKeyboard"
EndSection

Section "ServerFlags"
	Option	    "AllowMouseOpenFail" "yes"
EndSection

Section "InputDevice"
	Identifier  "Keyboard0"
	Driver      "kbd"
EndSection

Section "Device"
	Identifier  "Videocard0"
	Driver      "fbdev"
EndSection

Section "Screen"
	Identifier "Screen0"
	Device     "Videocard0"
	DefaultDepth     24
	SubSection "Display"
		Viewport   0 0
		Depth     24
	EndSubSection
EndSection
EOS
)

esxi_conf=$(cat << EOF >> /boot/AIP_xorgconf/vmware_xorg.conf.el5
# Xorg configuration created by pyxf86config

Section "ServerLayout"
	Identifier     "Default Layout"
	Screen      0  "Screen0" 0 0
	InputDevice    "Keyboard0" "CoreKeyboard"
EndSection

Section "InputDevice"
	Identifier  "Keyboard0"
	Driver      "kbd"
EndSection

Section "Device"
	Identifier  "Videocard0"
	Driver      "vmware"
EndSection

Section "Screen"
	Identifier "Screen0"
	Device     "Videocard0"
	DefaultDepth     24
	SubSection "Display"
		Viewport   0 0
		Depth     24
	EndSubSection
EndSection

EOF
)
}

Check_xorg () {
hv_conf_file=/boot/AIP_xorgconf/hyperv_xorg.conf.el5
hv_suse11_conf_file=/boot/AIP_xorgconf/hyperv_xorg.conf.suse11
ESXi_conf_file=/boot/AIP_xorgconf/vmware_xorg.conf.el5

if [ ! -d /boot/AIP_xorgconf ]; then
	mkdir /boot/AIP_xorgconf
fi
if [ -f $hv_conf_file -a -f $ESXi_conf_file -a -f $hv_suse11_conf_file ]; then
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Already exist xorg.conf." >> $LOG
else
	echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Not exist xorg.conf." >> $LOG
	xorg
	if [ -f $hv_conf_file -a -f $ESXi_conf_file ]; then
		echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Create xorg.conf successful." >> $LOG
		Sync_cmd
	else
		echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Create xorg.conf failed."	>> $LOG
	fi
fi
}

array_el5=(ata_piix mptbase mptsas mptspi pvscsi)
l_array_dracut=(ata_piix pvscsi hv_storvsc mptbase mptsas mptspi vmw_pvscsi vmxnet3  vmxnet  hv_vmbus hv_netvsc hv_blkvsc ahci libahci ata_generic virtio_scsi virtio_blk virtio_pci virtio_ring nvme xen-netfront xen-blkfront pata_acpi)

chk_exist_driver(){	#Does not exist xen driver in SLES11.x
for i in ${l_array_dracut[@]}
do
	find_file=`find /lib/modules/$(uname -r) -type f -name $i.ko* | wc -l`
	if [ $find_file -eq 1 ]; then
		if [[ "$i" =~ .*xen.* ]]; then
			xen_drv=`echo $i | sed s'/-/_/'`
			array_dracut_tmp+=($xen_drv)
		else
			array_dracut_tmp+=($i)
		fi
	fi
done

array_dracut_sort=( $( printf "%s\n" "${array_dracut_tmp[@]}" | sort ) )
array_dracut=`echo "${array_dracut_sort[@]}"`
echo "`date -u '+%m/%d/%Y %H:%M:%S.%3N'` ** Added to $initrd_file [$array_dracut]."	>> $LOG

}

Check_initrd
chk_exist_driver
Check_initramfstool

if [ $kernel_ver -le 2 ] ; then
	if [ -f /etc/redhat-release ]; then
		if [ -f /etc/modprobe.conf ]; then
			Check_driver_2
			Check_xorg
		else
			Check_dracut
			if [ $dracut_cmd -eq 1 ]; then
				Check_driver_3
				Check_xorg
			else
				mkinitrd_cmd
				Check_xorg 
			fi
		fi
	fi
fi

if [ $kernel_ver -ge 3 ]; then
	Check_dracut
	if [ $dracut_cmd -eq 1 ]; then
		Check_driver_3
		Check_xorg
	else
		if [ $initramfs_tools -eq 1 ]; then
			initramfs_tools
			Check_xorg
		else
			mkinitrd_cmd
			Check_xorg
		fi
	fi
fi

sleep 2

